/* oscillators.h.orc - written by Istvan Varga, 2002 */

; this file should be included from the orchestra header, after any
; header statements

; kfn   Cps2Fnum kcps, kbasfn
;
; Calculate table number (kfn) for the specified frequency (kcps),
; using base table number kbasfn. If kcps and kbasfn are set at
; i-time (or i-rate variables are used), the output can also be used
; at i-time with i(kfn).

	opcode Cps2Fnum, K, KK

kcps, kbasfn	xin
icps	limit i(kcps), 8.176, 12544
kfn	init int(log(icps) * 17.31234 + i(kbasfn) - 35.876)
kfn	limit log(kcps) * 17.31234 - 36.376, 0, 127
kfn	=  int(kfn + kbasfn + 0.5)
	xout kfn

	endop

; ar    PWMOsc kamp, kcps, kfn, kpw, kphs
;
; Oscillator with pulse width modulation.
;
; ar - the output variable.
;
; kamp - amplitude
; kcps - frequency
; kfn  - table number (should point to a band-limited sawtooth wave)
; kpw  - pulse width (in the range 0 to 1, use 0.5 for a square wave)
; kphs - phase offset (0 to 1)

	opcode PWMOsc, a, kkkkK

kamp, kcps, kfn, kpw, kphs	xin
a1	osciliktp kcps, kfn, kphs
a2	osciliktp kcps, kfn, kphs - kpw
aofs	interp 2 * kpw - 1, 0, 1
	xout kamp * (aofs + a1 - a2)

	endop

; ar    SawTriOsc kamp, kcps, kfn, kpw, kphs
;
; Sawtooth/triangle oscillator.
;
; ar - the output variable.
;
; kamp - amplitude
; kcps - frequency
; kfn  - table number (should point to a band-limited sawtooth wave)
; kpw  - waveform (0.01 to 0.99). 0.01: sawtooth down, 0.5: triangle,
;        0.99: sawtooth up
; kphs - phase offset (0 to 1)

	opcode SawTriOsc, a, kkkkK

kamp, kcps, kfn, kpw, kphs	xin
a1	osciliktp kcps, kfn, kphs
a2	osciliktp kcps, kfn, kphs - kpw
ascl	interp 0.25 * kamp / (kpw - kpw * kpw), 0, 1
	xout ascl * (a1 - a2)

	endop

